import 'dart:convert';
import 'package:youtube_explode_dart/youtube_explode_dart.dart';
import 'package:http/http.dart' as http;

class YoutubeHttpClientWithProxy extends YoutubeHttpClient {
  final String proxyUrl;

  YoutubeHttpClientWithProxy(this.proxyUrl);

  Uri factoryUriWithProxy(dynamic uri) {
    dynamic newUri;

    if (uri is String) {
      newUri = Uri.parse(proxyUrl + uri);
    } else if (uri is Uri) {
      newUri = Uri.parse(proxyUrl + uri.toString());
    } else {
      newUri = uri;
    }

    return newUri;
  }

  @override
  Future<http.Response> get(dynamic url,
          {Map<String, String>? headers = const {}, bool validate = false}) =>
      super.get(factoryUriWithProxy(url), headers: headers, validate: validate);

  @override
  Future<http.Response> post(Uri url,
          {Map<String, String>? headers,
          Object? body,
          Encoding? encoding,
          bool validate = false}) =>
      super.post(
        factoryUriWithProxy(url),
        headers: headers,
        body: body,
        encoding: encoding,
        validate: validate,
      );

  @override
  Future<http.Response> head(Uri url, {Map<String, String>? headers}) =>
      super.head(factoryUriWithProxy(url), headers: headers);

  @override
  Future<http.Response> put(Uri url,
          {Map<String, String>? headers, Object? body, Encoding? encoding}) =>
      super.put(factoryUriWithProxy(url),
          headers: headers, body: body, encoding: encoding);

  @override
  Future<http.Response> patch(Uri url,
          {Map<String, String>? headers, Object? body, Encoding? encoding}) =>
      super.patch(factoryUriWithProxy(url),
          headers: headers, body: body, encoding: encoding);

  @override
  Future<http.Response> delete(Uri url,
          {Map<String, String>? headers, Object? body, Encoding? encoding}) =>
      super.delete(factoryUriWithProxy(url),
          headers: headers, body: body, encoding: encoding);
  @override
  Future<http.StreamedResponse> send(http.BaseRequest request) async {
    request.headers['X-Requested-With'] = 'XMLHttpRequest';

    return super.send(request);
  }
}
